@@ -48,20 +48,23 @@ from mslib.msui import mpl_pathinteractor as mpl_pi
 
 
 class MapCanvas(basemap.Basemap):
-    """Derivative of mpl_toolkits.basemap, providing additional methods to
-       automatically draw a graticule and to redraw specific map elements.
+    """
+    Derivative of mpl_toolkits.basemap, providing additional methods to
+    automatically draw a graticule and to redraw specific map elements.
     """
 
-    def __init__(self, identifier=None, CRS=None, BBOX_UNITS=None,
+    def __init__(self, identifier=None, CRS=None, BBOX_UNITS=None, PROJECT_NAME=None,
                  appearance=None, **kwargs):
-        """New constructor automatically adds coastlines, continents, and
-           a graticule to the map.
+        """
+        New constructor automatically adds coastlines, continents, and
+        a graticule to the map.
 
         Keyword arguments are the same as for mpl_toolkits.basemap.
 
         Additional arguments:
         CRS -- string describing the coordinate reference system of the map.
         BBOX_UNITS -- string describing the units of the map coordinates.
+        PROJECT_NAME -- string with project name
 
         """
         # Coordinate reference system identifier and coordinate system units.
@@ -71,6 +74,9 @@ class MapCanvas(basemap.Basemap):
         else:
             self.bbox_units = getattr(self, "bbox_units", None)
 
+        self.project_name = PROJECT_NAME if PROJECT_NAME is not None else self.project_name \
+            if hasattr(self, "project_name") else None
+
         # Dictionary containing map appearance settings.
         if appearance is not None:
             param_appearance = appearance
@@ -127,12 +133,18 @@ class MapCanvas(basemap.Basemap):
 
         self.image = None
 
-        # Print CRS identifier into the figure.
+        # Print project name and CRS identifier into figure.
+        crs_text = ""
+        if self.project_name is not None:
+            crs_text += self.project_name
         if self.crs is not None:
-            if hasattr(self, "crs_text"):
-                self.crs_text.set_text(self.crs)
-            else:
-                self.crs_text = self.ax.figure.text(0, 0, self.crs)
+            if len(crs_text) > 0:
+                crs_text += "\n"
+            crs_text += self.crs
+        if hasattr(self, "crs_text"):  # update existing textbox
+            self.crs_text.set_text(crs_text)
+        else:
+            self.crs_text = self.ax.figure.text(0, 0, crs_text)
 
         if self.appearance["draw_graticule"]:
             try:
@@ -150,7 +162,8 @@ class MapCanvas(basemap.Basemap):
         self.identifier = identifier
 
     def set_axes_limits(self, ax=None):
-        """See Basemap.set_axes_limits() for documentation.
+        """
+        See Basemap.set_axes_limits() for documentation.
 
         This function is overridden in MapCanvas as a workaround to a problem
         in Basemap.set_axes_limits() that occurs in interactive matplotlib
@@ -167,7 +180,8 @@ class MapCanvas(basemap.Basemap):
         matplotlib.interactive(intact)
 
     def _draw_auto_graticule(self):
-        """Draw an automatically spaced graticule on the map.
+        """
+        Draw an automatically spaced graticule on the map.
         """
         # Compute some map coordinates that are required below for the automatic
         # determination of which meridians and parallels to draw.
@@ -279,7 +293,8 @@ class MapCanvas(basemap.Basemap):
                                                 labels=[0, 0, 0, 1], zorder=3)
 
     def set_graticule_visible(self, visible=True):
-        """Set the visibily of the graticule.
+        """
+        Set the visibily of the graticule.
 
         Removes a currently visible graticule by deleting internally stored
         line and text objects representing graticule lines and labels, then
@@ -315,7 +330,8 @@ class MapCanvas(basemap.Basemap):
 
     def set_fillcontinents_visible(self, visible=True, land_color=None,
                                    lake_color=None):
-        """Set the visibility of continent fillings.
+        """
+        Set the visibility of continent fillings.
         """
         if land_color is not None:
             self.appearance["colour_land"] = land_color
@@ -349,7 +365,8 @@ class MapCanvas(basemap.Basemap):
             self.ax.figure.canvas.draw()
 
     def set_coastlines_visible(self, visible=True):
-        """Set the visibility of coastlines and country borders.
+        """
+        Set the visibility of coastlines and country borders.
         """
         self.appearance["draw_coastlines"] = visible
         if visible and self.map_coastlines is None and self.map_countries is None:
@@ -385,7 +402,8 @@ class MapCanvas(basemap.Basemap):
             self.ax.figure.canvas.draw()
 
     def update_with_coordinate_change(self, kwargs_update=None):
-        """Redraws the entire map. This is necessary after zoom/pan operations.
+        """
+        Redraws the entire map. This is necessary after zoom/pan operations.
 
         Determines corner coordinates of the current axes, removes all items
         belonging the the current map and draws a new one by calling
@@ -489,8 +507,9 @@ class MapCanvas(basemap.Basemap):
     # self.urcrnry = axis[3]
 
     def imshow(self, X, **kwargs):
-        """Overloads basemap.imshow(). Deletes any existing image and
-           redraws the figure after the new image has been plotted.
+        """
+        Overloads basemap.imshow(). Deletes any existing image and
+        redraws the figure after the new image has been plotted.
         """
         if self.image is not None:
             self.image.remove()
@@ -522,8 +541,9 @@ class MapCanvas(basemap.Basemap):
         return x, y
 
     def gcpoints_path(self, lons, lats, del_s=100., map_coords=True):
-        """Same as gcpoints2, but for an entire path, i.e. multiple
-           line segments. lons and lats are lists of waypoint coordinates.
+        """
+        Same as gcpoints2, but for an entire path, i.e. multiple
+        line segments. lons and lats are lists of waypoint coordinates.
         """
         # use great circle formula for a perfect sphere.
         gc = pyproj.Geod(a=self.rmajor, b=self.rminor)
@@ -543,7 +563,9 @@ class MapCanvas(basemap.Basemap):
             # projection, gc.npts() returns lons that connect lon1 and lat2, not lon1 and
             # lon2 ... I cannot figure out why, maybe this is an issue in certain versions
             # of pyproj?? (mr, 16Oct2012)
-            lonlats = gc.npts(lons[i], lats[i], lons[i + 1], lats[i + 1], npoints)
+            lonlats = []
+            if npoints > 0:
+                lonlats = gc.npts(lons[i], lats[i], lons[i + 1], lats[i + 1], npoints)
             # The cylindrical projection of matplotlib is not periodic, that means that
             # -170 longitude and 190 longitude are not identical. The gc projection however
             # assumes identity and maps all longitudes to -180 to 180. This is no issue for
@@ -582,8 +604,9 @@ class MapCanvas(basemap.Basemap):
 
 
 class SatelliteOverpassPatch(object):
-    """Represents a satellite overpass on the top view map (satellite
-       track and, if available, swath).
+    """
+    Represents a satellite overpass on the top view map (satellite
+    track and, if available, swath).
     """
 
     # TODO: Derive this class from some Matplotlib actor class? Or create
@@ -610,7 +633,8 @@ class SatelliteOverpassPatch(object):
         self.draw()
 
     def draw(self):
-        """Do the actual plotting of the patch.
+        """
+        Do the actual plotting of the patch.
         """
         # Plot satellite track.
         sat = np.copy(self.sat)
@@ -651,15 +675,17 @@ class SatelliteOverpassPatch(object):
         self.map.ax.figure.canvas.draw()
 
     def update(self):
-        """Removes the current plot of the patch and redraws the patch.
-           This is necessary, for instance, when the map projection and/or
-           extent has been changed.
+        """
+        Removes the current plot of the patch and redraws the patch.
+        This is necessary, for instance, when the map projection and/or
+        extent has been changed.
         """
         self.remove()
         self.draw()
 
     def remove(self):
-        """Remove this satellite patch from the map canvas.
+        """
+        Remove this satellite patch from the map canvas.
         """
         if self.trackline is not None:
             for element in self.trackline:
